/*
  2025.11.17

  List all currently known USB-2200 modules and their PID & BID
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include "usb2200.h"

int main(void) {

  if(USB2200_ListDevice())
    printf("Open dir error\n");

  return 0;
}

